
//  Program:  student_avg
//  File:     main_student.cpp
//  Author:   Chuck Stewart
//  Purpose:  Compute student averages and output them alphabetically.
//    This demonstrates the use of classes and separate files.  It
//    also illustrates code reuse since the statistics functions from
//    the previous example are used again.
//  Use of the program:  The number of homeworks and number of tests 
//    are read first from the input file.  All students must have
//    the same number of each.  The weight to be given to the homework
//    average is  also read.  The weight given to the tests is 1 minus
//    the homework weight.  The program assumes that all homeworks are
//    equally weighted, all tests are equally weighted, and each
//    homework and test has the same maximum number of points.   The
//    output is formatted. 


#include <algorithm>
#include <fstream>
#include <iomanip>
#include <iostream>
#include <vector>

#include "Student.h"

unsigned int max( unsigned int x, unsigned int y )
{
  return x>y ? x : y;
}


//  Read information about a student, returning true if the 
//  information was read correctly and false otherwise.  The input
//  format is
//     id_number first_name last_name homeworks tests
//  numm_homeworks and num_tests ints are expected for the latter two.

bool
read_student( std::istream& in_str,
              unsigned int num_homeworks,
              unsigned int num_tests,
              Student & new_student )
{
  //  Start by resetting the new_student object in case it was
  //  previously used.
  new_student.reset();
  
  //  Start with the id number.  If there's no id, then end-of-file
  //  will likely have been reached, so silently return false.

  std::string input_id;
  if ( ! (in_str >> input_id ) )
    return false;

  //  Once we have an id number, any other failure in reading is
  //  treated as an error.  Start by reading the name.

  std::string input_first, input_last;
  if ( ! (in_str >> input_first >> input_last) )
    {
      std::cout << "Failed reading name for student "
                << input_id << std::endl;
      return false;
    }
  new_student.add_name_and_id( input_first, input_last, input_id );

  unsigned int i;
  int score;

  //  Read the homework scores
  int num_hw_read = 0;
  for ( i=0; i<num_homeworks && (in_str >> score); ++i )
    {
      new_student.add_hw( score );
      num_hw_read ++ ;
    }

  //  If we didn't get all scores, it is an error.
  if ( num_hw_read != num_homeworks )
    {
      std::cout << "Pre-mature end of file or invalid input reading "
           << "hw scores for " << input_id << std::endl;
      return false;
    }

  //  Read the test scores
  int num_test_read = 0;
  for ( i=0; i<num_tests && (in_str >> score); ++i )
    {
      new_student.add_test( score );
      num_test_read ++ ;
    }

  //  If we didn't get all scores, it is an error.

  if ( num_test_read != num_tests )
    {
      std::cout << "Pre-mature end of file or invalid input reading "
                << "test scores for"
                << input_id << std::endl;
      return false;
    }

  //  If we reach this point, everything is fine...
  return true;
}



int 
main( int argc, char* argv[] )
{
  //  Check the arguments and open the input and output streams

  if ( argc != 3 )
    {
      std::cerr << "Usage:\n  " << argv[0] << " infile-students outfile-grades\n";
      return 1;
    }

  std::ifstream in_str( argv[1] );
  if ( !in_str )
    {
      std::cerr << "Could not open " << argv[1] << " to read\n";
      return 1;
    }
  
  std::ofstream out_str( argv[2] );
  if ( !out_str )
    {
      std::cerr << "Could not open " << argv[2] << " to write\n";
      return 1;
    }

  //  Input the information about the number of homeworks, the number
  //  of tests and the weight of the homeworks.
  int num_homeworks, num_tests;
  double hw_weight;
  in_str >> num_homeworks >> num_tests >> hw_weight;

  // The main vector of students
  std::vector<Student> students;

  //  Read the students, one at a time.
  Student one_student;
  while( read_student( in_str, num_homeworks, num_tests, one_student ) )
    {
      students.push_back( one_student );
    }

  //  Compute the averages.  At the same time, determine the maximum
  //  name length.  
  unsigned int i;
  unsigned int max_length = 0;
  for ( i=0; i<students.size(); ++i )
    {
      students[i].compute_averages( hw_weight );
      max_length = max( max_length,
                        students[i].first_name().size()
                        + students[i].last_name().size() );
    }
  max_length += 2;   //  account for the output padding with ", "

  //  Sort the students alphabetically by name.
  std::sort( students.begin(), students.end(), less_names );

  //  Output a header...
  out_str << "\nHere are the student semester averages\n";
  const std::string header = "Name" + std::string( max_length-4, ' ')
    + "  HW  Test Final";
  const std::string underline( header.size(), '-' );
  out_str << header << '\n' << underline << std::endl;

  //  Output the students...
  for ( i=0; i<students.size(); ++i )
    {
      unsigned int length = students[i].last_name().size() + 
        students[i].first_name().size() + 2;
      students[i].output_name( out_str );
      out_str << std::string( max_length - length, ' ' ) << "  ";
      students[i].output_averages( out_str );
    }

  return 0;  // everything fine
}


