
//  File:     stats.cpp
//  Author:   Chuck Stewart
//  Purpose:  Statistics function definitions, including functions to
//    compute the average and standard deviation, the median, and the
//    mode.  These all work with integer valued vectors, which is a
//    limitation. 

#include <algorithm>
#include <cassert>
#include <cmath>
#include <vector>

//  Compute the mean (average) and standard deviation of a vector of
//  integer values.  At least two values must be there.
//
//  The vector of values is passed by constant reference to avoid the 
//  overhead of copying while not allowing the function to change it.
//  The parameters average and stddev are reference parameters so that
//  they can be "filled in" by the function.

void avg_and_stddev( const std::vector<int>& values,
                     double& average, double& stddev )
{
  int sum=0;           // Accumulation of the values
  unsigned int i;      // Counter variable  

  assert( values.size() > 1 );

  //  Compute the average

  for ( i=0; i<values.size(); ++i )
    {
      sum += values[i];
    }
  average = double(sum) / values.size();

  //  Compute the standard deviation

  double sum_sq_error = 0;
  for ( i=0; i<values.size(); ++i )
    {
      double error = values[i] - average;
      sum_sq_error += error * error;
    }
  stddev = sqrt( sum_sq_error / (values.size() - 1) );
}


//  Compute the median of an integer vector of values, returning the
//  median as the result of the function.  The vector must have at
//  least one value.
//
//  The vector is passed by value, meaning that a copy is made.  This
//  allows the function to change these values (by sorting them),
//  without changing the original function.

double compute_median( const std::vector<int> & values )
{
  assert( values.size() > 1 );
  std::vector<int> v( values );

  // Sort the values in the vector.  By default this is increasing
  // (technically, non-decreasing) order.

  std::sort( v.begin(), v.end() );

  //  Now, compute the median:

  unsigned int n = v.size();
  if ( n%2 == 0 )
    return double( v[n/2] + v[n/2-1] ) / 2;
  else
    return double( v[ n/2 ] );  // same as (n-1)/2 because n is odd
}


//  Compute the mode of an integer vector of values, returning the
//  mode as the result of the function.  The vector must have at
//  least one value.
//
//  The vector is passed by value, meaning that a copy is made.  This
//  allows the function to change these values (by sorting them),
//  without changing the original function.


int compute_mode( const std::vector<int> & values )
{
  // Make a copy and then sort the values in the vector.  By default
  // this is increasing (technically, non-decreasing) order.

  std::vector<int> v( values );
  std::sort( v.begin(), v.end() );

  //  On to computing the mode.

  int current_count = 1;
  int mode;
  int mode_count = 0;

  //  Loop invariants: 
  //  (a) current_count == number of occurrences of v[i-1] in the
  //      interval 0..i-1 of the v vector,
  //  (b) mode is the most frequently occurring score that is also less
  //      than v[i-1] in the interval 0..i-1 of the v vector
  //  (c) mode_count is the number of times mode occurs
  
  for ( unsigned int i=1; i<v.size(); ++i )
    {
      //  Same score
      if ( v[i] == v[i-1] )
        {
          current_count ++ ;
        }

      //  Change in the score
      else if ( current_count >= mode_count )
        {
          mode = v[i-1];
          mode_count = current_count;
          current_count = 1;
        }
      else
        {
          current_count = 1;
        }
    }

  //  Handle the possibility of the last score being the most
  //  frequent.  The statement of the loop invariant makes it clear
  //  that the loop ends without checking if the last value is the
  //  mode. 

  if ( current_count >= mode_count )
    {
      return v.back();       //  Last score
    }
  else
    {
      return mode;
    }
}
